@@ -1,6 +1,7 @@
 import configparser
 import inspect
 import os
+import re
 import pickle
 from datetime import datetime
 
@@ -19,9 +20,12 @@ from testit_adapter_pytest.utils import (
 
 class TestITListener(object):
     # TODO: need to rework for normal read
-    def __init__(self, testrun, url, private_token, configuration, proxy=None):
-        if testrun:
-            self.testrun_id = uuid_check(testrun)
+    def __init__(self, testrun, url, private_token, project, configuration, proxy=None, testrun_name=None):
+        if testrun or project:
+            if testrun:
+                self.testrun_id = uuid_check(testrun)
+            else:
+                self.project_id = uuid_check(project)
 
             if not url:
                 print('URL was not found!')
@@ -38,6 +42,10 @@ class TestITListener(object):
                 raise SystemExit
 
             self.configuration_id = uuid_check(configuration)
+
+            if testrun_name:
+                self.testrun_name = str(testrun_name)
+                print(f'The name of Test-run has been established: {testrun_name}')
         else:
             path = os.path.abspath('')
             root = path[:path.index(os.sep)]
@@ -53,9 +61,11 @@ class TestITListener(object):
 
             url = url_check(search_in_environ(parser.get('testit',
                                                          'url')))
-
             private_token = search_in_environ(parser.get('testit',
                                                          'privatetoken'))
+            self.configuration_id = uuid_check(
+                search_in_environ(parser.get('testit', 'configurationID')))
+
             try:
                 proxy = search_in_environ(parser.get('debug',
                                                      'testit_proxy'))
@@ -70,11 +80,10 @@ class TestITListener(object):
             else:
                 self.project_id = uuid_check(
                     search_in_environ(parser.get('testit', 'projectID')))
-                self.configuration_id = uuid_check(
-                    search_in_environ(parser.get('testit', 'configurationID')))
 
                 if parser.has_option('testit', 'testrun_name'):
                     self.testrun_name = parser.get('testit', 'testrun_name')
+                    print(f'The name of Test-run has been established: {self.testrun_name}')
         self.requests = Api(url, private_token, proxy=proxy)
 
     @pytest.hookimpl
@@ -124,9 +133,7 @@ class TestITListener(object):
 
                 item.test_external_id = self.param_attribute_collector(
                     item.function.test_external_id,
-                    item.own_markers,
-                    item.array_parametrize_mark_id,
-                    index) if hasattr(item,
+                    item.callspec.params) if hasattr(item,
                                       'array_parametrize_mark_id'
                                       ) else item.function.test_external_id
 
@@ -387,34 +394,21 @@ class TestITListener(object):
         if hasattr(item, 'array_parametrize_mark_id'):
             for link in item.function.test_links:
                 data['links'].append({})
-                data['links'][-1]['url'] = TestITListener.attribute_collector_links(
-                                                link,
-                                                'url',
-                                                item.own_markers,
-                                                item.array_parametrize_mark_id,
-                                                item.index)
+                data['links'][-1]['url'] = TestITListener.param_attribute_collector(
+                                                link['url'],
+                                                item.callspec.params)
                 if link['title']:
-                    data['links'][-1]['title'] = TestITListener.attribute_collector_links(
-                                                    link,
-                                                    'title',
-                                                    item.own_markers,
-                                                    item.array_parametrize_mark_id,
-                                                    item.index)
+                    data['links'][-1]['title'] = TestITListener.param_attribute_collector(
+                                                    link['title'],
+                                                    item.callspec.params)
                 if link['type']:
-                    data['links'][-1]['type'] = TestITListener.attribute_collector_links(
-                                                    link,
-                                                    'type',
-                                                    item.own_markers,
-                                                    item.array_parametrize_mark_id,
-                                                    item.index)
+                    data['links'][-1]['type'] = TestITListener.param_attribute_collector(
+                                                    link['type'],
+                                                    item.callspec.params)
                 if link['description']:
-                    data['links'][-1][
-                        'description'] = TestITListener.attribute_collector_links(
-                                            link,
-                                            'description',
-                                            item.own_markers,
-                                            item.array_parametrize_mark_id,
-                                            item.index)
+                    data['links'][-1]['description'] = TestITListener.param_attribute_collector(
+                                            link['description'],
+                                            item.callspec.params)
         else:
             data['links'] = item.function.test_links
 
@@ -423,10 +417,9 @@ class TestITListener(object):
         if not hasattr(item.function, 'test_title'):
             return None
         if hasattr(item, 'array_parametrize_mark_id'):
-            return TestITListener.param_attribute_collector(item.function.test_title,
-                                                            item.own_markers,
-                                                            item.array_parametrize_mark_id,
-                                                            item.index)
+            return TestITListener.param_attribute_collector(
+                item.function.test_title,
+                item.callspec.params)
         return item.function.test_title
 
     @staticmethod
@@ -436,9 +429,7 @@ class TestITListener(object):
         if hasattr(item, 'array_parametrize_mark_id'):
             return TestITListener.param_attribute_collector(
                 item.function.test_description,
-                item.own_markers,
-                item.array_parametrize_mark_id,
-                item.index)
+                item.callspec.params)
         return item.function.test_description
 
     @staticmethod
@@ -539,9 +530,7 @@ class TestITListener(object):
             if hasattr(item, 'array_parametrize_mark_id'):
                 data['autoTestName'] = TestITListener.param_attribute_collector(
                     item.function.test_displayname,
-                    item.own_markers,
-                    item.array_parametrize_mark_id,
-                    item.index)
+                    item.callspec.params)
             else:
                 data['autoTestName'] = item.function.test_displayname
         elif item.function.__doc__:
@@ -559,17 +548,33 @@ class TestITListener(object):
         return data
 
     @staticmethod
-    def param_attribute_collector(attribute, marks, parametrize_id, index):
-        for ID in parametrize_id:
-            if attribute[attribute.find('{') + 1:attribute.rfind('}')] in \
-                    marks[ID].args[0]:
-                return attribute.split('{')[0] + marks[ID].args[1][index][
-                    marks[ID].args[0].split(', ').index(attribute[
-                                                        attribute.find(
-                                                            '{') + 1:attribute.rfind(
-                                                            '}')])] + \
-                       attribute.split('}')[1]
-        return attribute
+    def param_attribute_collector(attribute, run_param):
+        result = attribute
+        param_keys = re.findall(r"\{(.*?)\}", attribute)
+        if len(param_keys) > 0:
+            for param_key in param_keys:
+                root_key = param_key
+                id_keys = re.findall(r'\[(.*?)\]', param_key)
+                if len(id_keys) == 0:
+                    base_key = root_key
+                    result = result.replace("{" + root_key + "}", str(run_param[base_key]))
+                elif len(id_keys) == 1:
+                    base_key = root_key.replace("[" + id_keys[0] + "]", "")
+                    id_key = id_keys[0].strip("\'\"")
+                    if id_key.isdigit() and int(id_key) in range(len(run_param[base_key])):
+                        val_key = int(id_key)
+                    elif id_key.isalnum() and not id_key.isdigit() and id_key in run_param[base_key].keys():
+                        val_key = id_key
+                    else:
+                        raise SystemExit(f"Not key: {root_key} in run parameters or other keys problem")
+                    result = result.replace("{" + root_key + "}", str(run_param[base_key][val_key]))
+                else:
+                    raise SystemExit("For type tuple, list, dict) support only one level!")
+        elif len(param_keys) == 0:
+            result = attribute
+        else:
+            raise SystemExit("Collecting parameters error!")
+        return result
 
     @staticmethod
     def mass_param_attribute_collector(attribute, marks, parametrize_id, index):
@@ -579,19 +584,6 @@ class TestITListener(object):
                 return marks[ID].args[1][index][param_id], param_id
         return attribute, None
 
-    @staticmethod
-    def attribute_collector_links(link, key, marks, parametrize_id, index):
-        for ID in parametrize_id:
-            if link[key][(link[key].find('{') + 1):(link[key].rfind('}'))] in \
-                    marks[ID].args[0]:
-                return link[key].split('{')[0] + marks[ID].args[1][index][
-                    marks[ID].args[0].split(', ').index(link[key][(
-                                                                          link[key].find('{') + 1):
-                                                                  (link[key].rfind(
-                                                                      '}'))])] + \
-                       link[key].split('}')[1]
-        return link[key]
-
     @staticmethod
     def form_tree_steps(item, tree_steps, stage):
         for scope in ('session', 'module', 'class', 'method'):
@@ -607,25 +599,28 @@ class TestITListener(object):
 
     @testit_adapter_pytest.hookimpl
     def add_link(self, link_url, link_title, link_type, link_description):
-        if not hasattr(self.item, 'result_links'):
-            self.item.result_links = []
-        self.item.result_links.append(
-            {
-                'url': link_url,
-                'title': link_title,
-                'type': link_type,
-                'description': link_description
-            })
+        if self.item:
+            if not hasattr(self.item, 'result_links'):
+                self.item.result_links = []
+            self.item.result_links.append(
+                {
+                    'url': link_url,
+                    'title': link_title,
+                    'type': link_type,
+                    'description': link_description
+                })
 
     @testit_adapter_pytest.hookimpl
     def add_message(self, test_message):
-        self.item.test_message = str(test_message)
+        if self.item:
+            self.item.test_message = str(test_message)
 
     @testit_adapter_pytest.hookimpl
     def add_attachments(self, attach_paths):
-        if not hasattr(self.item, 'test_attachments'):
-            self.item.test_attachments = []
-        self.item.test_attachments += self.load_attachments(attach_paths)
+        if self.item:
+            if not hasattr(self.item, 'test_attachments'):
+                self.item.test_attachments = []
+            self.item.test_attachments += self.load_attachments(attach_paths)
 
     @testit_adapter_pytest.hookimpl
     def load_attachments(self, attach_paths):
@@ -642,7 +637,7 @@ class TestITListener(object):
 
     @testit_adapter_pytest.hookimpl
     def add_parameters(self, parameters):
-        if not hasattr(self.item, 'test_parameters'):
+        if self.item and not hasattr(self.item, 'test_parameters'):
             self.item.test_parameters = {}
             for key, parameter in parameters.items():
                 self.item.test_parameters[key] = str(parameter)
